""" plugin_dbui.py

    Controllers expose by the plugin:
    
        index
        database
        configuration

    Author: R. Le Gac

"""

import os


API = """
App.csvUrl = '/%s/plugin_dbui/csv';
App.config = %s;
App.name = '%s';
App.REMOTE_API = {
    'url': '/%s/plugin_dbui/call',
    'type': 'remoting',
    'actions': %s 
};"""


MATHJAX = """
MathJax.Hub.Config({
    extensions: ["tex2jax.js"],
    jax: ["input/TeX", "output/HTML-CSS"],
    tex2jax: {
        inlineMath: [ ['$','$'], ["\\(","\\)"] ],
        displayMath: [ ['$$','$$'], ["\\[","\\]"] ],
        processEscapes: true
    },
    "HTML-CSS": { availableFonts: ["TeX"] }
});
"""

def call():
    """Action to handle the Ext.Direct protocol.
    
    """
    return directSvc()


def csv():
    """Action returning the content of a table as a CSV file.
    Foreign keys are replaced by the parent column.
    
    """
    import gluon.contenttype
    import plugin_dbui as dbui

    tablename = request.vars['tableName']

    response.headers['Content-Type'] = gluon.contenttype.contenttype('.csv')
    response.headers['Content-Disposition'] = 'attachment; filename=%s.csv;' % tablename
    
    table = db[tablename]
    query = table.id > 0
    
    # replace foreign key by parent fields
    # and build the query to resolve foreign keys
    if dbui.is_table_with_foreign_fields(table):
    
        fields = []
        query = ((query) & (dbui.get_where_query(table)))
        
        for field in table:
        
            if dbui.is_foreign_field(field):
                k_table, k_field, k_key = dbui.get_foreign_field(field)
                fields.append(db[k_table][k_field])
                
            else:
                fields.append(field)
    else:
        fields = [table.ALL]

    # interrogate the database    
    rows = db(query).select(*fields)
    return str(rows)
    

def dbui_conf():
    """Action returning the javascript script configuring
    the plugin dbui: application name, store definition,...
    
    """
    import plugin_dbui as dbui
    from gluon.contrib import simplejson as json

    di = {}
    
    # build the dictionary required by Ext.direct
    # containing the definition of the remote procedure
    for (k, f) in directSvc.procedures.iteritems():
        action, method = k.split('.')

        if action == dbui.DBUI:
            nargs = f.func_code.co_argcount
        else:
            nargs = f.__func__.func_code.co_argcount - 1
            
        if action not in di:
            di[action] = []
            
        di[action].append({'name': method, 'len': nargs})

    # the definition of the models
    config = {'models': {}, 
              'stores': {}, 
              'treeNodes': [],
              'viewport': None}
    
    for table in dbui.get_all_tables(db):
        config['models'][table._tablename] = dbui.to_model(table) 
      
    # the stores configuration (static, for each table,...)
    # NOTE: the interface require a store for all tables including alias.
    # The only way to extract them is to scan the attributes list of 
    # the DAL since the method db.tables() or any variant return 
    # tables but not the alias one
    config['stores'].update(plugins.dbui.static_stores)
    for table in dbui.get_all_tables(db):
        cfg = dbui.to_jsonstore(table)
        config['stores'][cfg['storeId']] = cfg 
    
    # the tree nodes configuration for the TreeStore
    config['treeNodes'] = dbui.to_treeNodes()
    
    # the viewport configuration
    config['viewport'] = dbui.to_viewport()
    
    # fill the javascript template
    app = request.application
    script = API % (app, 
                    json.dumps(config),
                    app,
                    app, 
                    json.dumps(di))
    
    # return the response as a javascript
    response.headers['Content-Type'] = 'text/javascript'
    return script


def debug():
    """Action to run the plugin in debug mode
    Load the javascript source code for all libraries.
    Active the debug mode on the server side.

    EXAMPLES
    
        http://localhost:8000/myapp/plugin_dbui/debug
        http://localhost:8000/myapp/plugin_dbui/debug?script=foo

    URL OPTIONS

        script 
            run the javascript foo in the framework defined by the plugin.
            The scripts are stored in the directory defined by the plugin
            configuration app_script_dir. 

    """
    import plugin_dbui as dbui

    # check the presence of the ExtJS and MathJax plugins
    plugin = plugins.dbui

    # css files
    response.files.extend(dbui.get_file_paths(plugin.extjs_css, ext='.css'))
    response.files.extend(dbui.get_file_paths(plugin.dbui_css, ext='.css'))
    response.files.extend(dbui.get_file_paths(plugin.ace_css, ext='.css'))
    response.files.extend(dbui.get_file_paths(plugin.app_css, ext='.css'))

    # debug version of the javascript libraries
    response.files.extend(dbui.get_file_paths(plugin.ace_libmin, ext='.js'))
    
    response.files.extend(dbui.get_file_paths(plugin.mathjax_libmin, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.extjs_js, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.dbui_js, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.app_js, ext='.js'))

    # language files
    response.files.extend(dbui.get_file_paths(plugin.extjs_lg, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.dbui_lg, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.app_lg, ext='.js'))
    
    # URL for dbui configuration script
    response.files.append(plugin.dbui_conf)
    
    # main script which can be defined in many different ways
    pscript = dbui.get_script_path(plugin)
    response.files.append(pscript)

    # switch ON the debug mode on the server side
    session.debug = True

    # select the view    
    response.view = 'plugin_dbui/index.html'
    
    # return the plugin configuration parameter as well as the response 
    return dict(plugin=plugin)
    

def index():
    """Default Action to run the plugin
    Load compressed version of all libraries.
    
    EXAMPLES
        
        http://localhost:8000/myapp/plugin_dbui
        http://localhost:8000/myapp/plugin_dbui?script=foo
        
    URL OPTIONS

        script
            run the javascript foo in the framework defined by the plugin.
            The scripts are stored in the directory defined by the plugin
            configuration app_script_dir.

    """
    import plugin_dbui as dbui

    plugin = plugins.dbui

    # css files
    response.files.extend(dbui.get_file_paths(plugin.extjs_css, ext='.css'))
    response.files.extend(dbui.get_file_paths(plugin.dbui_css, ext='.css'))
    response.files.extend(dbui.get_file_paths(plugin.ace_css, ext='.css'))
    response.files.extend(dbui.get_file_paths(plugin.app_css, ext='.css'))

    # compressed version of the javascript libraries
    response.files.extend(dbui.get_file_paths(plugin.ace_libmin, ext='.js'))

    response.files.extend(dbui.get_file_paths(plugin.mathjax_libmin, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.extjs_libmin, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.dbui_libmin, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.app_libmin, ext='.js'))
    
    # language files
    response.files.extend(dbui.get_file_paths(plugin.extjs_lg, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.dbui_lg, ext='.js'))
    response.files.extend(dbui.get_file_paths(plugin.app_lg, ext='.js'))
    
    # URL for dbui configuration script
    response.files.append(plugin.dbui_conf)
    
    # main script which can be defined in many different ways
    pscript = dbui.get_script_path(plugin)
    response.files.append(pscript)

    # switch off the debug mode on the server side
    session.debug = False

    # return the plugin configuration parameter as well as the response 
    return dict(plugin=plugin)


def mathjax_conf():
    """Action returning the js configuration script for the MathJax library.
    
    """
    return MATHJAX


def status():
    return dict(request=request, response=response, session=session)


def versions():
    """Return the versions of the different part of the code.
    
    """
    import plugin_dbui as dbui
    
    my_style = \
    """
        margin-left: 20px; 
        font-size: 10px;
    """
    return BEAUTIFY(dbui.get_versions(), _style=my_style)
