""" the modifier class

@author: R. Le Gac

"""

from gluon.storage import Storage
from gluon.tools import PluginManager


class Modifier(object):
    """Base class to construct Modifier tools.
    
    """

    def __init__(self, dpname, tablename=None):
        """Create the L{gluon.storage.Storage} in the plugin dbui to house 
        modifier information.
         
            - The path to access to the Storage is defined by the C{dpname}
              and C{tablename}.
            - The C{Storage} is linked to the class attribute C{data}. 
            - The C{Storage} can be accessed outside the modifier
              in the following way:
        
                  >>> p = PluginManager('dbui')
                  >>> modifier = p.dbui[dpname]
                    
              or when the tablename is defined:
        
                  >>> p = PluginManager('dbui')
                  >>> modifier = p.dbui[dpname][tablename]
                  
            - The C{Storage} contains the C{extjs} key to store
              configuration parameter of the associate Ext JS widget.
        
        @type dpname: str
        @param dpname: name of the data point.
        
        @type tablename: str
        @param tablename: name of the database table.
        
        """    
        p = PluginManager('dbui')
        
        if tablename:        
            if dpname not in p.dbui:
                p.dbui[dpname] = {}

            if tablename not in p.dbui[dpname]:
                p.dbui[dpname][tablename] = Storage(extjs={}) 
                
            self.data = p.dbui[dpname][tablename]
            
        else:
            if dpname not in p.dbui:
                p.dbui[dpname] = Storage(extjs={})
                
            self.data = p.dbui[dpname]
 
        
    def configure(self, **extjs):
        """Set the configuration parameters for the associated Ext JS widget.
        
        @note:
            - Existing value of the configuration parameters are replaced by
              those defined in the keyword arguments.
        
            - The keyword argument C{plugins} contains a list of plugins.
              The current list of C{plugins} are extend with the new ones
              defined by the keyword argument C{plugins}.
               
            - A plugin is defined by a string containing is C{pType}.
            
            - A plugin is defined by a dictionary containing the C{pType}
              and the plugin configuration options::
              
                  {'ptype': foo, 'option1': val,...}
        
            - The widget associated by default is replaced by a new one
              when the keyword argument C{xtype} is used.
            
        """
        if 'plugins' in extjs:
            
            plugins = extjs['plugins']
            
            if isinstance(plugins, str):
                plugins = [plugins]
                
            if 'plugins' in self.data.extjs:
                self.data.extjs['plugins'].extend(plugins)
                
            else:
                self.data.extjs['plugins'] = list(plugins)
            
            del extjs['plugins']
            
        self.data.extjs.update(extjs)