""" the store modifier.

@author: R. Le Gac

"""

from gluon.storage import Storage
from gluon.tools import PluginManager
from helper import get_store_id
from modifier import Modifier


MODIFIER_STORES = 'modifier_stores'


class AddStore(object):
    """ A Ext JS store is associated to each database table. Their configuration
    are derived from the table models using the L{converter.to_jsonstore}.
    
    The Ext JS configuration options of all stores are kept in a 
    persistent dictionary of the plugin_dbui. 
    The configuration options are transported
    to the client using the C{plugin_dbui/dbui_conf} action.
    
    From time to time, it is useful to defined static store, 
    like C{Ext.data.ArrayStore}. This tool allows to defined them
    and to add their configuration to the persistent dictionary.
    Thus they can be retrieve by the client in the standard way.
    
    This approach is generic and any store can be added to the 
    persistent dictionary.
    
    """ 
    
    def __init__(self, name, **kwargs):
        """ Create the configuration option of an Ext JS Store.
        
        @type name: str
        @param name: name for the store
        
        The keyword arguments contains the configuration options for
        the store.
        For an C{Ext.data.ArrayStore} the keyword C{fields} and C{data}
        have to be defined. For more details see the ExtJS documentation.
        
        @note: 
        The key associated to this store in the persistent dictionary
        is the C{storeId}. Its value is derived from the store name, 
        I{i.e} C{nameStore}.
        
        """
        static_stores = PluginManager('dbui').dbui.static_stores
    
        storeId = get_store_id(name)
        kwargs['storeId'] = storeId
        static_stores[storeId] = kwargs

    
class StoreModifier(Modifier):
    """Modifier to customize the store associated to a table.
    
    """
    
    def __init__(self, tablename):
        """Initialize the store modifier associated to the database
        table C{tablename}.
        
        The data structure of the store modifier can be access 
        outside the class:
        
            >>> p = PluginManager('dbui')
            >>> modifier = p.dbui['modifier_stores'][tablename]

        or internally via the attribute C{data}.
        
        The keys of the C{data} structure are:
        
            - C{extjs} (dict) Ext JS configuration options for the store widget.
            
            - C{order_by} (list of gluon.dal.Field) list of fields to order
            the content of the store
            
        @type tablename: str
        @param tablename: name of the database table

        """
        Modifier.__init__(self, MODIFIER_STORES, tablename)

        if 'orderby' not in self.data:
            self.data.orderby = []


    def orderby(self, *fields):
        """Order the store according to the list of gluon.dal.field.
        
        """
        self.data.orderby.extend(fields)
        
    
    def set_buffered(self, **kwargs):
        """ The store is buffered. 
        The method set the following configuration options:
        
            - autoLoad = False
            - buffered = True
            - clearOnPageLoad = True
            - pageSize = 25
            - leadingBufferZone = 0
            - trailingBufferZone = 0
        
        They can be overwritten using the keyword arguments.
        
        The grid buffering is activate when the store is buffered.
        That means that only 5 pages are kept in the cache. Therefore
        this is a good option to manipulate almost infinite set of data.
        
        @param kwargs: Any configuration parameter of the C{Ext.data.Store}.
        Those related to the buffering are C{buffered}, C{clearOnPaqgeLoad},
        C{leadingBufferZone}, C{pageSize}, C{purgePageCount} 
        and C{trailingBufferZone}. 
        
        """
        di = dict(autoLoad=False,
                  buffered=True,
                  clearOnPageLoad=True,
                  pageSize=25,
                  leadingBufferZone=0,
                  trailingBufferZone=0)
        
        di.update(kwargs)
        
        self.configure(**di)
        