/**
 * The form panel is an Ext.form.Panel linked to a database table.
 *
 * It contains field widget for the database table fields and
 * two buttons *Action* and *Reset*.
 *
 * The store attribute is equal the table name when the widget
 * is configured. It is replaced by an App.data.DirectStore
 * in the initialization phase of the component.
 *
 * It also comes with a set of methods to
 * create, duplicate, destroy and update record in the database table.
 * Therefore this component can be used as a classic form, a row editor
 * for a grid, a record browser, ....
 *
 * The action associated to the button is set via the #setAction method.
 *
 */
Ext.define('App.form.Panel', {

    extend: 'Ext.form.Panel',
    alias: 'widget.xform',

    /**
     * @property {Ext.Button}
     * The action button of the form
     */
    buttonAction: null,

    /**
     * @property {Ext.Button}
     * The reset button of the form
     */
    buttonReset: null,

    /**
     * @cfg {Ext.data.Store} store (required)
     * The Ext.data.Store that the form should use.
     * The store has to be defined since the actions rely on it.
     */
    store: null,

    // Predefined configuration options
    autoScroll: true,
    bodyStyle: 'padding:5px 5px 0',
    buttons: [{
        formBind: true,
        itemId: 'buttonAction',
        text: 'Action'
    }, {
        itemId: 'buttonReset',
        text: 'Reset'
    }],
    defaults: {anchor: '100%'},
    defaultType: 'textfield',
    frame: true,

    // private properties to keep track of current action parameters
    currentAction: null,
    currentRecord: null,

    // keep track of the index of the selected record in the store
    // useful to add a new record before or after the selected one
    currentIndex: undefined,

    // private properties for internationalization
    textCreate: 'Create',
    textDestroy: 'Delete',
    textDuplicate: 'Duplicate',
    textReset: 'Reset',
    textUpdate: 'Update',

    textCreateErrorAction: 'The new record(s) is destroyed.',
    textCreateFailed: 'Create failed...',
    textDbAction: 'The database is not modified.',
    textDeleteErrorAction: 'The record(s) is kept.',
    textDeleteFailed: 'Delete failed...',
    textError: 'Error...',
    textServerError: 'Internal server error.',
    textStoreError: 'the store is undefined !!!',
    textUpdateErrorAction: 'The update record(s) is revert to its original values.',
    textUpdateFailed: 'Update failed...',

    // private method require by the ExtJs component model
    initComponent: function () {

        var i,
            toolbar;

        // initialise the base class
        this.callParent(arguments);

        // short cuts
        toolbar = this.getDockedItems()[0];
        this.buttonAction = toolbar.getComponent('buttonAction');
        this.buttonReset = toolbar.getComponent('buttonReset');

        // button events listeners
        this.buttonAction.on('click', this.doAction, this);
        this.buttonReset.on('click', this.softReset, this);

        // link the form to the data store
        this.store = App.getStore(this.store);
        this.store.getProxy().on('exception', this.onProxyException, this);
        this.store.on('write', this.softReset, this);

        // set the default action: create
        this.setAction('create');
        this.buttonReset.setText(this.textReset);
    },

    /**
     * Add a new record. It is inserted after the selected one
     * when the attribute #currentIndex is defined.
     *
     * @param {Ext.data.Model} record
     */
    addRecord: function (record) {

        "use strict";

        // add the new record after the selected one
        if (this.currentIndex !== undefined) {
            this.store.insert(this.currentIndex + 1, record);

        // add the new record at the end of the store
        } else {
            this.store.add(record);
        }

        // reset the index
        this.currentIndex = undefined;
    },

    // private method required by the ExtJS component model
    beforeDestroy: function () {

        this.buttonAction.un('click', this.doAction, this);
        this.buttonReset.un('click', this.softReset, this);

        this.store.un('exception', this.onStoreException, this);
        this.store.un('write', this.softReset, this);

        this.callParent(arguments);
    },

    /**
     * Enable or disable all fields of the form
     * @param {Object} boolean
     */
    disableFields: function (bool) {

        "use strict";

        var fields = this.getForm().getFields();

        fields.each(function (field) {
            field.setDisabled(bool);
            return true;
        });

    },

    /**
     * Execute the #currentAction on the store.
     * The current action is set by the #setAction method.
     *
     * **Note:** Raise an error when the store is not defined.
     */
    doAction: function () {

        "use strict";

        var form = this.getForm(),
            model = this.store.getProxy().getModel(),
            newRecord;

        if (!form.isValid()) {
            return;
        }

        if (!this.store) {
            throw new Error(this.textStoreError);
        }

        switch (this.currentAction) {

        case 'create':
            newRecord = model.create();
            this.updateRecord(newRecord);
            this.addRecord(newRecord);
            break;

        case 'destroy':
            this.store.remove(this.currentRecord);
            break;

        case 'duplicate':
            newRecord = model.create();
            this.updateRecord(newRecord);
            this.addRecord(newRecord);
            break;

        case 'update':
            this.currentRecord.beginEdit();
            this.updateRecord(this.currentRecord);
            this.currentRecord.endEdit();
            break;
        }

        if (!this.store.autoSync) {
            this.store.sync();
        }
    },

    /**
     * Hard reset of the form.
     * The field values are reset to the default values.
     *
     */
    hardReset: function () {

        "use strict";

        var fields = this.getForm().getFields();

        fields.each(function (field) {
            field.originalValue = field.getInitialConfig('value');
            field.setValue(field.originalValue);
        });
    },

    /**
     * Process the proxy exception.
     * See Ext.data.proxy.Direct for the description of the arguments.
     *
     * @param {Ext.data.proxy.Proxy} proxy
     * @param {Object} response
     * @param {Ext.data.Operation} operation
     * @param {Object} eOpts
     *
     */
    onProxyException: function (proxy, response, operation, eOpts) {

        "use strict";

        var errors,
            key,
            msg = "";

        // invalid response from the server, HTPP 400, 500
        if (response instanceof Ext.direct.ExceptionEvent) {
            Ext.Msg.alert(this.textError, response.xhr.responseText);
            return;
        }

        // valid answer from the server, HTTP 200
        // but something went wrong in the server validation process, ...
        if (response instanceof Ext.direct.RemotingEvent) {

            if (!response.result.success) {
                errors = response.result.errors;

                if (Ext.isString(errors)) {
                    msg = errors;

                } else {
                    for (key in errors) {
                        if (errors.hasOwnProperty(key) && errors[key]) {

                            msg += errors[key];
                            msg += "<br>";
                        }
                    }
                }

                Ext.Msg.alert(this.textError, msg);
                return;
            }
        }
    },

    /**
     * Set the action associated to the form.
     * The record is load into the form and the text of the button is
     * properly set.
     *
     * @param {String} action
     * Valid action are create, destroy, duplicate, update and view.
     * @param {Ext.data.Record} record
     */
    setAction: function (action, record) {

        "use strict";

        var form = this.getForm(),
            table,
            tableId;

        this.buttonReset.show();
        this.buttonAction.show();

        this.disableFields(false);

        this.currentAction = action;
        this.currentRecord = record;

        // force orginalValues to be equal to the load data
        // this is required by the softReset method.
        form.trackResetOnLoad = true;

        switch (action) {

        case 'create':
            this.hardReset();
            this.buttonAction.setText(this.textCreate);
            break;

        case 'destroy':
            this.buttonAction.setText(this.textDestroy);
            form.loadRecord(record);
            break;

        case 'duplicate':
            this.buttonAction.setText(this.textDuplicate);
            this.hardReset();
            table = App.getTableName(this.store);
            tableId = App.encodeField(table, 'id');
            delete record.data[tableId];
            form.loadRecord(record);
            break;

        case 'update':
            this.buttonAction.setText(this.textUpdate);
            form.loadRecord(record);
            break;

        case 'view':
            this.buttonReset.hide();
            this.buttonAction.hide();
            form.loadRecord(record);
            this.disableFields(true);
            break;
        }
    },

    /**
     * Soft reset of the form.
     * The field values are reset to those originally loaded.
     */
    softReset : function () {

        "use strict";

        this.getForm().reset();
    },

    /**
     * Update the selected record with the value of the form.
     * @param {Ext.data.Model} record
     */
    updateRecord: function (record) {

        "use strict";

        var combo,
            field,
            fields = [],
            i,
            items,
            rec,
            value;

        function pushToFields(key, value) {
            fields.push(value);
        }

        // get the list of dirty fields
        items = this.getForm().getFields();
        items.each(function (item, index, len) {
            if (item.getXType() !== 'compositefield') {
                fields.push(item);
            }
        });

        // include dirty fields embedded in composite fields
        items.each(function (item, index, len) {
            if (item.getXType() === 'compositefield') {
                item.items.eachKey(pushToFields);
            }
        });

        // update the record
        // take care of special treatment required by date and combobox
        for (i = 0; i < fields.length; i += 1) {

            field = fields[i];
            value = field.getValue();

            switch (field.getXType()) {

            // We convert the date object according to a string defined
            // by the Ext.form.field.Date property format.
            // NOTE: by default in the json encoding, the date object
            // is converted as string using iso format YYYY-MM-DDTHH:MM:SS.
            // However, the string expected by the database depends on
            // the date type: date, datetime or time. The format of the
            // Ext.form.field.Date, used by the interface, is defined in the
            // property format.

            case 'datefield':
                if (Ext.isDate(value)) {
                    value = value.format(field.format);
                }
                break;

            // For foreign key, the record contains the valueField
            // as well as the displayField. The default action update
            // the valueField but note the display field.
            // The next lines append the displayField

            case 'xcombobox':
                combo = field;
                rec = combo.findRecord(combo.valueField, combo.getValue());
                record.set(combo.displayField, rec.get(combo.displayField));
                break;
            }

            record.set(field.getName(), value);
        }
    }
});
