/**
 * The field AceEditor is a wrapping of the
 * [ACE editor](http://http://ace.c9.io/#nav=about)
 * It can be embedded in Ext.form.Panel.
 *
 * @experimental
 * @since 0.4.15.0
 *
 */
Ext.define('App.form.field.AceEditor', {

    extend: 'Ext.form.field.Base',
    alias: 'widget.xaceeditorfield',

    /**
     * @cfg {Number}
     * The font size in pixels
     */
    fontSize: 10,

    /**
     * @cfg {String}
     * The programming language javascript, python, ....
     */
    language: 'python',

    /**
     * @cfg {Number} editorHeight (required)
     * The height of the editor widget
     */
    editorHeight: 150,

    /**
     * @cfg {String}
     */
    theme: 'crimson_editor',

    // private property
    editor: null,
    valueBeforeRender: undefined,

    // private method requests by the component model of ExtJS
    initComponent: function (config) {

        // Hack to get the size of the editor render properly
        // The width is to small when it is not specified.
        this.minWidth = this.minWidth || 258;

        this.callParent(arguments);
    },

    // private method requests by the component model of ExtJS
    beforeDestroy: function () {

        Ext.destroyMembers(this, 'editor');

        this.callParent(arguments);
    },

    /**
     * Return the value of this field.
     *
     * @return {String}
     *
     */
    getValue: function () {

        "use strict";

        if (this.editor) {
            return this.editor.getValue();
        }

        return undefined;
    },

    /**
     * Render the editor field.
     *
     * @param {Ext.Element} ct The container to render to.
     * @param {Object} position The position in the container to render to.
     *
     */
    onRender: function (ct, position) {

        this.callParent(arguments);

        // wrap an element around the input field
        this.resizeEl = this.positionEl = this.wrap = this.bodyEl.wrap();

        // the editor require that the width an height are defined
        this.wrap.setWidth(this.wrap.getWidth());
        this.wrap.setHeight(this.editorHeight);

        // instantiate the editor
        this.editor = ace.edit(this.wrap.id);
        this.editor.getSession().setMode('ace/mode/' + this.language);
        this.editor.setFontSize(this.fontSize);

        if (this.theme) {
            this.editor.setTheme('ace/theme/' + this.theme);
        }

        // load the value
        this.editor.setValue(this.valueBeforeRender);
        this.editor.clearSelection();
        this.editor.moveCursorTo(0, 0);

        // hide the default input field
        this.bodyEl.setVisibilityMode(Ext.dom.Element.DISPLAY).hide();
    },

    /**
     * Resize the component
     *
     * @param {Number} w
     * @param {Number} h
     *
     */
    onResize: function (w, h) {

        this.wrap.setSize(w - this.getLabelWidth(), h);
        if (this.editor) {
            this.editor.resize();
        }

        this.callParent(arguments);
    },

    /**
     * Set the value for this field.
     *
     * @param {String} value
     *
     */
    setValue: function (value) {

        this.callParent(arguments);

        if (this.editor) {
            this.editor.setValue(value);
            this.editor.clearSelection();
            this.editor.moveCursorTo(0, 0);

            this.valueBeforeRender = undefined;

        // Keep track of the value.
        // It will be used at the first rendering of the editor
        } else {
            this.valueBeforeRender = value;
        }
    }
});