/**
 * The App.form.field.ComboBoxMaster and the App.form.field.ComboBoxSlave
 * are two standard Ext.form.field.ComboBox.
 * linked together in such a way that the values available in the slave
 * depends on the selected value in the master.
 *
 * Both are bind to the unique store, the
 * {@link App.form.field.ComboBoxMaster#refStore refStore}.
 * It defines the relation between master and slave values. It has to contain the
 * displayField and the valueField for the master and for the slave.
 *
 * The slave ComboBox has two additional configuration parameters:
 *
 * - {@link App.form.field.ComboBoxSlave#masterItemId masterItemId}.
 *   It is the itemId of the master widget.
 *   It is required to link the slave and the master widget in order to
 *   implement the logic between them.
 *
 * - {@link App.form.field.ComboBoxSlave#masterValueField masterValueField}.
 *   The field name containing the valueField used by the master.
 *
 * Several slaves can be bind to the same master.
 *
 * @since 0.6.0.0
 *
 */
Ext.define('App.form.field.ComboBoxMaster', {

    extend: 'Ext.form.field.ComboBox',
    alias: 'widget.xcomboboxmaster',

    /**
     * @cfg {Ext.data.Store/String} refStore (required)
     * The reference store shared by the master and by the slave.
     * It contains the relation between the master and slave values.
     * It has to contain the displayField and the valueField for the master
     * and for the slave.
     */
    refStore: null,

    // Predefined setting
    // Works in auto completion mode when the user type value.
    // In any case the value should belongs to the list.
    editable: true,
    forceSelection: true,
    queryMode: 'local',
    triggerAction: 'all',

    // private properties for internationalization
    textSelect: 'Select a ',

    // private method require by the ExtJs component model
    initComponent: function () {

        // defined the empty text from the field label
        if (this.fieldLabel && !this.emptyText) {
            this.emptyText = this.textSelect + this.fieldLabel + " ...";
        }

        // instantiate the reference store which contains
        // the relation between the master and the slave value
        this.refStore = App.getStore(this.refStore);

        // build the local store containing master values
        // Value are static and never change.
        this.store = Ext.create('Ext.data.ArrayStore', {
                fields: [this.displayField, this.valueField]
        });

        // initialise the base class
        this.callParent(arguments);

        // fill the local store from the reference store
        this.loadStore();
    },

    /**
     * Load the local store with the value from the reference one
     *
     */
    loadStore: function () {

        "use strict";

        if (this.refStore.isFiltered() || !this.refStore.getTotalCount()) {

            this.refStore.restoreWhere();

            this.refStore.load({
                callback: this.loadData,
                scope: this
            });

        } else {
            this.loadData(this.refStore.getRange());
        }
    },

    /**
     * Load the data form the reference store to the local store
     *
     * @param {Ext.data.Model[]} records
     *
     */
    loadData: function (records) {

        "use strict";

        var data=[],
            i,
            keys=[],
            master,
            value;

        for (i = 0; i < records.length; i += 1) {

            master = records[i].get(this.displayField);

            if (keys.indexOf(master) === -1) {
                keys.push(master);
                value = records[i].get(this.valueField);
                data.push([master, value]);
            }
        }

        this.store.loadData(data);
    }
});