/**
 * The counter part of the App.form.field.ComboBoxMaster.
 *
 * @since 0.6.0.0
 *
 */
Ext.define('App.form.field.ComboBoxSlave', {

    extend: 'Ext.form.field.ComboBox',
    alias: 'widget.xcomboboxslave',

    /**
     * @cfg {String} masterValueField (required)
     * The name of field containing master values.
     */
    masterValueField: undefined,

    /**
     * @cfg {String} masterItemId (required)
     * The itemId of the master ComboBox.
     */
    masterItemId: undefined,

    /**
     * @cfg {Ext.data.Store/String} refStore (required)
     * The reference store shared by the master and by the slave.
     * It contains the relation between the master and slave values.
     * It has to contain the displayField and the valueField for the master
     * and for the slave.
     */
    refStore: null,

    // Predefined setting
    // Works in auto completion mode when the user type value.
    // In any case the value should belongs to the list.
    editable: true,
    forceSelection: true,
    queryMode: 'local',
    triggerAction: 'all',

    // private properties for internationalization
    textSelect: 'Select a ',

    // private method require by the ExtJs component model
    initComponent: function () {

        // defined the empty text from the field label
        if (this.fieldLabel && !this.emptyText) {
            this.emptyText = this.textSelect + this.fieldLabel + " ...";
        }

        // instantiate the reference store which contains
        // the relation between the master and the slave value
        this.refStore = App.getStore(this.refStore);

        // build the local store containing slave and master values
        this.store = Ext.create('Ext.data.ArrayStore', {
                fields: [
                    this.masterValueField,
                    this.displayField,
                    this.valueField
                ]
        });

        // initialise the base class
        this.callParent(arguments);

        // fill the local store from the reference store
        this.loadStore();

        // the slave is disable at start up
        this.setDisabled(true);

        // Bind the slave to the master widget.
        // Can only be done when widgets exists.
        this.on('afterrender', this.bindToMaster, this, {single: true});
    },

    /**
     * Handler binding the master and the slave comboBoxes.
     * It is triggered by the afterrender event.
     *
     */
    bindToMaster: function () {

        "use strict";

        // Find the master widget
        // Assume that it is above in the widget stack.
        // Not the optimal way but failed to use this.up(path) ???
        var path = 'xcomboboxmaster[itemId=' + this.masterItemId + ']',
            master = this.getRefOwner().query(path)[0];

        master.on('select', this.onMasterSelect, this);
    },

    /**
     * Load the local store with the value from the reference one
     *
     */
    loadStore: function () {

        "use strict";

        if (this.refStore.isFiltered() || !this.refStore.getTotalCount()) {

            this.refStore.restoreWhere();

            this.refStore.load({
                callback: this.loadData,
                scope: this
            });

        } else {
            this.loadData(this.refStore.getRange());
        }
    },

    /**
     * Load the slave data from the reference store to the local store.
     * The pair of values (master, slave) is unique.
     *
     * @param {Ext.data.Model[]} records
     *
     */
    loadData: function (records) {

        "use strict";

        var data=[],
            i,
            key,
            keys=[],
            master,
            slave,
            value;

        for (i = 0; i < records.length; i += 1) {

            master = records[i].get(this.masterValueField);
            slave = records[i].get(this.displayField);

            key = master+slave;

            if (keys.indexOf(key) === -1) {
                keys.push(key);
                value = records[i].get(this.valueField);
                data.push([master, slave, value]);
            }
        }

        this.store.loadData(data);
    },

    /**
     * Select the authorized slave values.
     * When the number of authorized value is equal to 1, the
     * value is loaded in the slave.
     *
     * @param {Ext.form.field.ComboBox} combo
     * @param {Ext.data.Model[]} records
     * @param {Object} eOpts
     */
    onMasterSelect: function (combo, records, eOpts) {

        "use strict";

        var value = records[0].get(this.masterValueField);

        // clear previous filter
        this.store.clearFilter(true);
        this.reset();

        // filter the local store
        this.store.filter(this.masterValueField, value);

        // enable the widget
        this.setDisabled(false);

        // load the value when there is only one
        if (this.store.getCount() === 1) {
            this.setValue(this.store.getAt(0).get(this.valueField));
        }
    },

    /**
     * Reset and disabled the slave ComboBox
     */
    reset: function () {
        this.setDisabled(true);
        this.callParent(arguments);
    }
});