/**
 * Extend the default type for Ext.grid.column in order to render JSON object.
 *
 * @since 0.4.15.0
 *
 */
Ext.define('Ext.grid.JsonColumn', {

    extend: 'Ext.grid.column.Column',
    alias: 'widget.jsoncolumn',

    constructor: function (cfg) {

        this.renderer = JSON.stringify;
        this.callParent(arguments);
    }
});

/**
 * The Dict widget is an Ext.grid.PropertyGrid which is associated
 * to a JSON database field. It can be embedded in Ext.form.Panel.
 *
 * The JSON object is a dictionary. It can only contains boolean,
 * date, float and string objects.
 *
 * The default structure of the dictionary can be defined via
 * the configuration option #dictCfg.
 * It is possible to add / delete key value pairs.
 *
 * @experimental
 * @since 0.4.15.0
 *
 */
Ext.define('App.form.field.Dict', {

    extend: 'Ext.form.field.Base',
    alias: 'widget.xdictfield',

    /**
     * @cfg {Object}
     * The default configuration for the dictionary, e.g.
     *
     *     {
     *         "(name)": "My Object",
     *         "Created": new Date(Date.parse('10/15/2006')),
     *         "Available": false,
     *         "Version": .01,
     *         "Description": "A test object"
     *     }
     *
     */
    dictCfg: {},

    /**
     * @cfg {String[]}
     * The labels of the columns to be shown in the grid widget.
     */
    headers: null,

    /**
     * @cfg {Boolean}
     * The columns header is hidden when true.
     */
    hideHeader: false,

    /**
     * @cfg {Boolean}
     * Dictionary keys can be added or deleted when true.
     */
    modifyKeys: false,

    // private short cuts
    grid: null,
    menu: null,
    record: null,

    // private properties for internationalization
    textAddMenu: 'Add',
    textAddTitle: 'Please enter the new key',
    textDestroyMenu: 'Delete',
    textDestroyTitle: 'Are you sure to destroy the key',
    textErrorTitle: 'Error ...',
    textErrorMsg: 'The key already exists!',
    textUpdateMenu: 'Update',
    textUpdateTitle: 'Modify the key',

    // private method requests by the component model of ExtJS
    initComponent: function () {

        var columns,
            i,
            headerContainer;

        // property grid
        this.grid = Ext.create('Ext.grid.property.Grid', {
            hideHeaders: this.hideHeaders,
            source: Ext.apply({}, this.dictCfg)
        });

        // dedicated column headers
        if (this.headers) {

            headerContainer = this.grid.getDockedItems('headercontainer')[0];
            columns = headerContainer.getGridColumns();

            for (i = 0; i < columns.length; i += 1) {
                columns[i].setText(this.headers[i]);
            }
        }

        // context menu
        this.grid.on('cellcontextmenu', this.onCellContextMenu, this);
        this.grid.on('containercontextmenu', this.onContainerContextMenu, this);
        this.grid.on('headercontextmenu', this.onHeaderContextMenu, this);

        if (this.modifyKeys) {

            this.menu = Ext.create('Ext.menu.Menu', {
                items: [{
                    text: this.textAddMenu,
                    iconCls: 'xaction-create',
                    itemId: 'add',
                    handler: this.onAddKey,
                    scope: this
                }, '-', {
                    text: this.textUpdateMenu,
                    iconCls: 'xaction-update',
                    itemId: 'update',
                    handler: this.onUpdateKey,
                    scope: this
                }, '-', {
                    text: this.textDestroyMenu,
                    iconCls: 'xaction-destroy',
                    itemId: 'destroy',
                    handler: this.onDeleteKey,
                    scope: this
                }]
            });
        }

        // instantiate the base class
        this.callParent(arguments);
    },

    // private method requests by the component model of ExtJS
    beforeDestroy: function () {

        this.grid.un('cellcontextmenu', this.onCellContextMenu, this);
        this.grid.un('containercontextmenu', this.onContainerContextMenu, this);
        this.grid.un('headercontextmenu', this.onHeaderContextMenu, this);

        Ext.destroyMembers(this, 'grid');
        Ext.destroyMembers(this, 'menu');

        this.callParent(arguments);
    },

    /**
     * Return the value of this field.
     *
     * @return {Object} the content of the dictionary.
     *
     */
    getValue: function () {

        "use strict";
        return Ext.apply({}, this.grid.getSource());
    },

    /**
     * Add a key to the dictionary.
     */
    onAddKey: function () {

        "use strict";

        // prompt the user to get the key name
        Ext.Msg.prompt(this.textAddTitle, "", function (btn, text) {

            if (btn === 'ok') {

                // check that the key is unique
                if (this.grid.getSource().hasOwnProperty(text)) {
                    Ext.Msg.alert(this.textErrorTitle, this.textErrorMsg);
                    return;
                }
                // add the new key
                this.grid.setProperty(text, null, true);
            }
        }, this);
    },

    /**
     * Show the context menu to modify key when right clicking on the first row.
     *
     * @param {Ext.view.Table} table
     * @param {HTMLElement} td
     * @param {Number} cellIndec
     * @param {Ext.data.Model} record
     * @param {HTMLElement} tr
     * @param {Number} rowIndex
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onCellContextMenu: function (table, td, cellIndex, record, tr, rowIndex, event, eOpts) {

        "use strict";
        event.stopEvent();

        if (this.modifyKeys && cellIndex === 0) {
            this.record = record;

            this.menu.getComponent('destroy').setDisabled(false);
            this.menu.getComponent('update').setDisabled(false);

            this.menu.showAt(event.getXY());
        }
    },

    /**
     * Show the context menu when the grid is empty.
     * In that case only the add option is allowed.
     *
     * @param {Ext.view.View} view
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onContainerContextMenu: function (view, event, eOpts) {

        "use strict";
        event.stopEvent();

        if (this.modifyKeys) {
            this.menu.getComponent('destroy').setDisabled(true);
            this.menu.getComponent('update').setDisabled(true);

            this.menu.showAt(event.getXY());
        }
    },

    /**
     * Delete a key of the dictionary.
     */
    onDeleteKey: function () {

        "use strict";

        var prop = this.record.get("name");

        Ext.Msg.confirm(this.textDestroyTitle, prop, function (btn) {
            if (btn === "yes") {
                this.grid.removeProperty(prop);
            }
        }, this);
    },

    /**
     * Inhibit the context menu when the user right click on the header
     *
     * @param {Ext.grid.header.Container} headerContainer
     * @param {Ext.grid.column.Column} column
     * @param {Ext.EventObject} event
     * @param {HTMLElement} html
     * @param {Object} eOpts
     */
    onHeaderContextMenu: function (headerContainer, column, event, html, eOpts) {

        "use strict";
        event.stopEvent();
    },

    /**
     * Render the widget.
     *
     * @param {Ext.Element} ct The container to render to.
     * @param {Object} position The position in the container to render to.
     *
     */
    onRender: function (ct, position) {

        this.callParent(arguments);

        // wrap an element around the input field
        this.resizeEl = this.positionEl = this.wrap = this.bodyEl.wrap();
        this.grid.render(this.wrap);

        // hide the default input field
        this.bodyEl.setVisibilityMode(Ext.dom.Element.DISPLAY).hide();
    },

    /**
     * Resize the component
     *
     * @param {Number} w
     * @param {Number} h
     *
     */
    onResize: function (w, h) {

        this.grid.setSize(w - this.getLabelWidth(), h);
        this.callParent(arguments);
    },

    /**
     * Update a key.
     */
    onUpdateKey: function () {

        "use strict";

        var key = this.record.get("name"),
            value = this.record.get("value");

        // prompt the user to modify the key
        Ext.Msg.prompt(this.textUpdateTitle, "", function (btn, text) {

            if (btn === 'ok') {

                // check that the key is unique
                if (this.grid.getSource().hasOwnProperty(text)) {
                    Ext.Msg.alert(this.textErrorTitle, this.textErrorMsg);
                    return;
                }
                // update the key
                this.grid.removeProperty(key);
                this.grid.setProperty(text, value, true);
            }
        }, this, false, key);

    },

    /**
     * Set the value for the dictionary.
     *
     * @param {Object} value
     * A key, value object which is loaded in the dictioanry.
     *
     */
    setValue: function (value) {

        var source  = Ext.apply({}, value, this.dictCfg);
        this.grid.setSource(source);

        return this.callParent([source]);
    }
});