/**
 * The field list is an Ext.grid.Panel which is associated
 * to the database list field. It can be embedded in Ext.form.Panel.
 *
 * **Note** :The list can only contains string.
 *
 * **Note** : The onRender handle can be removed using the approach
 * describe
 * [here](http://stackoverflow.com/questions/6153362/how-to-create-custom-extjs-form-field-component)
 * (see rixo)
 *
 * @experimental
 * @since 0.4.15.0
 *
 */
Ext.define('App.form.field.List', {

    extend: 'Ext.form.field.Base',
    alias: 'widget.xlistfield',

    /**
     * @cfg {string}
     * The label of the column to be shown in the grid widget.
     */
    header: 'Element',

    /**
     * @cfg {boolean}
     * Hide the column header when true.
     */
    hideHeader: false,

    /**
     * @cfg{boolean}
     * Empty string are not copied in the list return by the #getValue method.
     */
    ignoreEmptyRows: true,

    /**
     * @cfg {number}
     * The minimum number of element displayed in the list.
     * The grid is filled with empty string when values are missing.
     * Empty string can be ignored when getting the list using
     * the property #ignoreEmptyRows.
     *
     */
    minimumRows: 3,

    // private short cuts
    grid: null,
    menu: null,
    record: null,
    rowIndex: undefined,
    store: null,

    // private properties for internationalization
    textAdd: 'Insert element',
    textDestroy: 'Delete element',
    textDestroyTitle: 'Are you sure to destroy the element',
    textGoDown: 'Move down',
    textGoUp: 'Move up',

    // private method requests by the component model of ExtJS
    initComponent: function () {

        var cellEditing;

        // Hack to get the size of the grid render properly.
        // The width is to small when it is not specified.
        this.minWidth = this.minWidth || 258;

        // instantiate the local store
        this.store = Ext.create('Ext.data.ArrayStore', {
            fields: [{name: 'element'}]
        });

        // instantiate the cellEditing plugin
        cellEditing = Ext.create('Ext.grid.plugin.CellEditing', {
            clicksToEdit: 1
        });

        // instantiate the grid
        this.grid = Ext.create('Ext.grid.Panel', {
            columns: [{
                dataIndex: 'element',
                editor: Ext.create('Ext.form.field.Text', {}),
                flex: 1,
                itemId: 'element',
                sortable: false,
                text: this.header
            }],
            hideHeaders: this.hideHeader,
            plugins: [cellEditing],
            selType: 'cellmodel',
            store: this.store
        });

        // instantiate the context menu
        this.menu = Ext.create('Ext.menu.Menu', {
            items: [{
                text: this.textAdd,
                iconCls: 'xaction-create',
                itemId: 'add',
                handler: this.onAddRow,
                scope: this
            }, '-', {
                text: this.textGoUp,
                iconCls: 'xaction-go-up',
                itemId: 'goUp',
                handler: this.onGoUp,
                scope: this
            }, {
                text: this.textGoDown,
                iconCls: 'xaction-go-down',
                itemId: 'goDown',
                handler: this.onGoDown,
                scope: this
            }, '-', {
                text: this.textDestroy,
                iconCls: 'xaction-destroy',
                itemId: 'destroy',
                handler: this.onDeleteRow,
                scope: this
            }]
        });

        this.grid.on('containercontextmenu', this.onContainerContextMenu, this);
        this.grid.on('itemcontextmenu', this.onItemContextMenu, this);

        this.callParent(arguments);
    },

    // private method requests by the component model of ExtJS
    beforeDestroy: function () {

        this.grid.un('containercontextmenu', this.onContainerContextMenu, this);
        this.grid.un('itemcontextmenu', this.onItemContextMenu, this);

        Ext.destroyMembers(this, 'grid');
        Ext.destroyMembers(this, 'menu');
        Ext.destroyMembers(this, 'store');

        this.callParent(arguments);
    },

    /**
     * Return the value of this field
     * Empty string are not copied in the list when #ignoreEmptyRows is true.
     *
     * @return {Array}
     *
     */
    getValue: function () {

        "use strict";

        var i,
            li = [],
            records = this.store.getRange(),
            value;

        for (i = 0; i < records.length; i += 1) {

            value = records[i].get("element");

            if (!this.ignoreEmptyRows || value) {
                li.push(value);
            }
        }

        return li;
    },

    /**
     * Add an element to the list below the selected one.
     */
    onAddRow: function () {

        "use strict";

        var record = this.store.getProxy().getModel().create();

        record.set("element", "");
        this.store.insert(this.rowIndex + 1, record);
    },

    /**
     * Show the context menu.
     *
     * @param {Ext.view.View} view
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onContainerContextMenu: function (view, event, eOpts) {

        "use strict";
        event.stopEvent();

        this.menu.getComponent('destroy').setDisabled(true);
        this.menu.getComponent('goDown').setDisabled(true);
        this.menu.getComponent('goUp').setDisabled(true);

        this.rowIndex = -1;
        this.menu.showAt(event.getXY());
    },

    /**
     * Delete the selected element of the list.
     */
    onDeleteRow: function () {

        "use strict";

        var value = this.record.get("element");

        Ext.Msg.confirm(this.textDestroyTitle, value, function (btn) {
            if (btn === 'yes') {
                this.grid.getStore().remove(this.record);
            }
        }, this);
    },

    /**
     * Move the selected element down.
     */
    onGoDown: function () {

        "use strict";

        var recordA,
            recordB,
            valueA,
            valueB;

        recordA = this.store.getAt(this.rowIndex);
        recordB = this.store.getAt(this.rowIndex + 1);

        valueA = recordA.get('element');
        valueB = recordB.get('element');

        recordA.set('element', valueB);
        recordB.set('element', valueA);
    },

    /**
     * Move the selected element up.
     */
    onGoUp: function () {

        "use strict";

        var recordA,
            recordB,
            valueA,
            valueB;

        recordA = this.store.getAt(this.rowIndex);
        recordB = this.store.getAt(this.rowIndex - 1);

        valueA = recordA.get('element');
        valueB = recordB.get('element');

        recordA.set('element', valueB);
        recordB.set('element', valueA);
    },

    /**
     * Show the row context menu.
     *
     * @param {Ext.view.View} view
     * @param {Ext.data.Model} record
     * @param {HTMLElement} item
     * @param {Number} index
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onItemContextMenu: function (view, record, item, index, event, eOpts) {

        "use strict";

        event.stopEvent();

        this.record = record;
        this.rowIndex = index;

        this.menu.showAt(event.getXY());

        this.menu.getComponent('destroy').setDisabled(false);
        this.menu.getComponent('goDown').setDisabled(false);
        this.menu.getComponent('goUp').setDisabled(false);

        if (index === 0) {
            this.menu.getComponent('goUp').setDisabled(true);
        }

        if (index === this.grid.getStore().getCount() - 1) {
            this.menu.getComponent('goDown').setDisabled(true);
        }

    },

    /**
     * Render the field.
     *
     * @param {Ext.Element} ct The container to render to.
     * @param {Object} position The position in the container to render to.
     *
     */
    onRender: function (ct, position) {

        this.callParent(arguments);

        // wrap an element around the input field
        this.resizeEl = this.positionEl = this.wrap = this.bodyEl.wrap();
        this.grid.render(this.wrap);

        // hide the default input field
        this.bodyEl.setVisibilityMode(Ext.dom.Element.DISPLAY).hide();
    },

    /**
     * Resize the component
     *
     * @param {Number} w
     * @param {Number} h
     *
     */
    onResize: function (w, h) {

        this.grid.setSize(w - this.getLabelWidth(), h);
        this.callParent(arguments);
    },

    /**
     * Set the value for this field.
     *
     * @param {Array} value
     * The content of the list.
     */
    setValue: function (value) {

        var data = [],
            i;

        // convert the list in a list of list
        if (value && value instanceof Array) {

            for (i = 0; i < value.length; i += 1) {
                data.push([value[i]]);
            }

            if (value.length < this.minimumRows) {
                for (i = value.length; i < this.minimumRows; i += 1) {
                    data.push([]);
                }
            }

        } else {

            for (i = 0; i < this.minimumRows; i += 1) {
                data.push([]);
            }
        }

        // replace the content of the store with the new data set
        this.store.loadData(data, false);

        this.callParent(arguments);
    }
});