/**
 * The Filter is a {@link Ext.form.FieldSet set of fields}
 * It containing all the logic to filter the content of the grid
 * according to the value defined in the fields.
 *
 */
Ext.define('App.grid.Filter', {

    extend: 'Ext.form.FieldSet',
    alias: 'widget.xgridfilter',

    // Private object containing where clause related to the filter request.
    // this is used when running with a buffered store.
    filterConditions: {},

    // Private reference to the grid
    grid: null,

    // Predefined setting
    //
    // NOTE:
    // the enableKeyEvents is mandatory for Ext.form.field.Text
    //
    defaults: {
        anchor: '100%',
        enableKeyEvents: true
    },

    // private method requests by the component model of ExtJS
    initComponent: function () {

        // initialise the base class
        this.callParent(arguments);

        // associate handlers to fields
        this.items.each(function (item) {

            if (!item.isFormField) {
                return;
            }

            switch (item.getXType()) {

                // catch a change of value for comboBox
                case 'xcombobox':
                    item.on('select', this.onChange, this);
                    break;

                case 'xcomboboxmaster':
                    item.on('select', this.onChange, this);
                    break;

                case 'xcomboboxslave':
                    item.on('select', this.onChange, this);
                    break;

                // catch a change of date
                case 'datefield':
                    item.on('select', this.onChange, this);
                    break;

                // catch the key pressed enter for other fields
                // wait that user finish to type to run the handler
                default:
                    item.on('keyup', this.onChange, this, {buffer: 500});
                    break;
            }
        }, this);
    },

    // private method requires by the Ext JS component model
    beforeDestroy: function () {

        //purge handlers associate to fields defined in the initComponent
        this.items.each(function (item) {

            if (!item.isFormField) {
                return;
            }

            switch (item.getXType()) {

                // catch a change of value for comboBox
                case 'xcombobox':
                    item.un('select', this.onChange, this);
                    break;

                case 'xcomboboxmaster':
                    item.un('select', this.onChange, this);
                    break;

                case 'xcomboboxslave':
                    item.un('select', this.onChange, this);
                    break;

                // catch a change of date
                case 'datefield':
                    item.un('select', this.onChange, this);
                    break;

                // catch the key pressed enter for other fields
                // wait that user finish to type to run the handler
                default:
                    item.un('keyup', this.onChange, this, {buffer: 500});
                    break;
            }
        }, this);

        // remove private reference
        this.grid = null;

        // base class
        this.callParent(arguments);
    },

    /**
     * Bind the grid filter to the grid
     * @param {Ext.grid.Panel} grid
     */
    bind: function (grid) {

        "use strict";
        var buttons = grid.query('xgridbuttonexpertmenu'),
            buttonReset;

        this.grid = grid;

        // clicking the reset button of the grid reset the filter's fields.
        if (buttons && buttons[0] && buttons[0].menu) {
            buttonReset = buttons[0].menu.getComponent('buttonReset');
            buttonReset.on('click', this.onReset, this);
        }
    },

    /**
     * Create a filter for non buffered store
     *
     * @param {Ext.form.Field} field
     *
     */
    createLocalFilter: function (field) {

        "use strict";

        var t, f,
            filter,
            operator = field.filterOperator,
            property,
            value = field.getValue();

        // build the field identifier for the store
        [t, f] = field.name.substr(1, field.name.length-2).split('.');
        property = App.encodeField(t, f);

        // polish the pair operator, value
        // contains are not case sensitive → 'i'
        switch (operator) {

            case "==":
                operator = '=';
                break;

            case "contains":
                value = new RegExp(".*" + value + ".*", 'i');
                operator = undefined;
                break;
        }

        // build the filter
        filter = {
            id: field.name,
            property: property,
            operator: operator,
            value: value
        };

        return filter;
    },

    /**
     * Setup filter conditions for non buffered store
     *
     * @param {Ext.form.Field} field
     *
     */
    localFiltering: function (field) {

        "use strict";

        var store = this.grid.getStore(),
            filter = store.filters.getByKey(field.name),
            value  = field.getValue();

        // Clear an existing filter
        if (filter && field.filterType === 'string' && value === "") {

            store.removeFilter(filter);
            return;
        }

        // update an existing filter
        if (filter) {

            switch (field.filterOperator) {

                case "contains":
                    value = new RegExp(".*" + value + ".*");
                    break;
            }
            filter.setValue(value);
            store.filter();
            return;
        }

        // create a new filter
        filter = this.createLocalFilter(field);
        store.addFilter(filter);
    },

    /**
     * Handler to catch a change in field value and to filter the store.
     *
     * @param {Ext.form.Field} field
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     */
    onChange: function (field, event, eOpts) {

        "use strict";

        if (this.grid.remoteAction) {
            this.remoteFiltering(field);

        } else {
            this.localFiltering(field);
        }
    },

    /**
     * Handler to reset the filter and the store content
     */
    onReset: function () {

        "use strict";

        // reset the field values
        this.items.each(function (item) {
            if (item.isFormField) {
                item.reset();
            }
        });

        // reset the grid
        this.grid.reset();
    },

    /**
     * Setup filter conditions for a buffered store
     *
     * @param {Ext.form.Field} field
     *
     */
    remoteFiltering: function (field) {

        "use strict";

        var conditions = [],
            i,
            k,
            store = this.grid.getStore(),
            value,
            where;

        // get the field value
        value = field.getValue();

        // handle date using the proper format
        if (field.xtype === 'datefield') {
            if (value instanceof Date) {
                value = value.format(field.format);
            } else {
                return;
            }
        }

        //  update the condition dictionary
        where = field.name + " " + field.filterOperator + " '" + value + "'";

        if (value === '') {
            delete this.filterConditions[field.name];
        } else {
            this.filterConditions[field.name] = where;
        }

        // build the complete where clause
        if (store.initialWhere) {
            for (i = 0; i < store.initialWhere.length; i += 1) {
                conditions.push(store.initialWhere[i]);
            }
        }

        for (k in this.filterConditions) {
            if (this.filterConditions.hasOwnProperty(k)) {
                conditions.push(this.filterConditions[k]);
            }
        }

        // push new condition in the where clause of the store
        if (conditions.length) {
            store.extraParams.where = conditions;

        // go back to the initial where setting when conditions are empty
        } else {
            store.restoreWhere();
        }

        // load the data
        // NOTE
        // Clear the page cache to fix some overlap problem (ExtJS 4.2.1)
        // Some records are not over written and correspond to
        // the previous filtering. Clearing the buffer solve this issue.
        // Copy from the method Ext.data.Store.reload
        store.data.clear(true);
        store.loadPage(1);
    }
});