/**
 * The plugin adding the paging tool bar.
 * It contains the widget to navigate between page and a slider
 * to fixed the number of rows per page.
 *
 */
Ext.define('App.grid.plugin.Paging', {

    extend: 'Ext.AbstractPlugin',
    alias: 'plugin.pGridPaging',

    /*
     * @cfg {Number} increment
     * The slider increment
     */
    increment: 20,
    /*
     * @cfg {Number} pageSize
     */
    pageSize: 10,

    // short cuts
    bbar: null,
    slider: null,

    // Private property for internationalisation
    textPlus: 'Plus',
    textSlider: 'Entries per page',

    /**
     * Initialize the plugin
     *
     * @param {App.grid.Panel} grid
     *
     */
    init: function (grid) {

        "use strict";

        var slider,
            store = grid.getStore();

        // keep track of the parent
        this.setCmp(grid);

        // create the paging bar tool bar
        this.bbar = Ext.create('Ext.toolbar.Paging', {
            dock: 'bottom',
            displayInfo: false,
            items: ['-', this.textSlider, {
                increment: this.increment,
                minValue: this.pageSize,
                width: 150,
                xtype: 'slider'
            }, '-', '->', {
                grid: grid,
                text: this.textPlus,
                xtype: 'xgridbuttonexpertmenu'
            }],
            store: store
        });

        // insert the tool bar in the grid
        grid.insertDocked(0, this.bbar);

        // Set-up the remote action parameters
        store.pageSize = this.pageSize;
        store.getProxy().pageParam = 'page';
        grid.remoteAction = true;

        // short cut
        this.slider = slider = this.bbar.query('slider')[0];

        // event related to the slide
        this.slider.on('change', this.onSliderChange, this, {buffer: 2000});

        // events related to the store
        store.on('filterchange', this.onStoreChange, this);
        store.on('load', this.onStoreChange, this);
        store.on('write', this.onStoreChange, this);
    },

    /**
     * Destroy the plugin
     */
    destroy: function () {

        "use strict";
        var store = this.getCmp().getStore();

        this.slider.un('change', this.onSliderChange, this, {buffer: 2000});

        // events related to the store
        store.un('filterchange', this.onStoreChange, this);
        store.un('load', this.onStoreChange, this);
        store.un('write', this.onStoreChange, this);
    },

    /**
     * Fire when the slider value is change by the user.
     * The page size is updated and the first page loaded.
     *
     * @param {Ext.slider.Multi} slider
     *
     * @param {Number} newValue
     * The new value which the slider has been changed too.
     *
     * @param {Ext.slider.Thumb} thumb
     * the thumb that was changed.
     *
     * @param {Object} eOpts
     *
     */
    onSliderChange: function (slider, newValue, thumb, eOpts) {

        "use strict";
        var grid = this.getCmp();

        grid.getStore().pageSize = newValue;
        this.bbar.moveFirst();
    },

    /**
     * Fire when the store is changed (load, write, filterchange)
     * The maximum value for the slider is updated and the first page is loaded.
     */
    onStoreChange: function () {

        "use strict";
        var store = this.getCmp().getStore(),
            count = store.getTotalCount(),
            value;

        if (count) {
            value = ((count/this.increment).toFixed() + 1) * this.increment;

        } else {
            value = this.pageSize;
        }

        this.slider.setMaxValue(value);
    }
});