/**
 * The plugin instantiating the context menu for the App.grid.plugin.RowEditor.
 *
 * Display the context menu when the user right click on a row.
 * The content of the menu allows to manipulate the App.grid.plugin.RowEditor.
 *
 * @uses App.grid.plugin.RowEditor
 *
 */
Ext.define('App.grid.plugin.RowEditorContextMenu',  {

    extend: 'Ext.AbstractPlugin',
    alias: 'plugin.pGridRowEditorContextMenu',

    // private short cut
    menu: null,

    // private properties for internationalization
    textAdd: 'Add',
    textCreate: 'Create',
    textDestroy: 'Delete',
    textDuplicate: 'Duplicate',
    textUpdate: 'Update',
    textView: 'View',

    /**
     * Initialize the plugin.
     *
     * @param {App.grid.Panel} grid
     */
    init: function (grid) {

        "use strict";

        var rowEditor = grid.getPlugin('rowEditor');

        this.setCmp(grid);

        // protection
        if (!rowEditor) {
            throw new Error('no grid row editor !!!');
        }

        this.menu = Ext.create('Ext.menu.Menu', {
            items : [{
                text: this.textAdd,
                iconCls: 'xaction-create',
                handler: rowEditor.onAddRow,
                scope: rowEditor
            }, '-', {
                text: this.textDuplicate,
                iconCls: 'xaction-duplicate',
                handler: rowEditor.onDuplicateRow,
                scope: rowEditor
            }, {
                text: this.textUpdate,
                iconCls: 'xaction-update',
                handler: rowEditor.onEditRow,
                scope: rowEditor
            }, {
                text: this.textView,
                iconCls: 'xaction-view',
                handler: rowEditor.onViewRow,
                scope: rowEditor
            }, '-', {
                text: this.textDestroy,
                iconCls: 'xaction-destroy',
                handler: rowEditor.onDeleteRow,
                scope: rowEditor
            }]
        });

        grid.on({
            'containercontextmenu': this.onContainerContextMenu,
            'headercontextmenu': this.onHeaderContextMenu,
            'itemcontextmenu': this.onItemContextMenu,
            scope: this
        });
    },

    /**
     * Destroy the plugin.
     */
    destroy: function () {

        "use strict";

        var grid = this.getCmp();

        grid.un('containercontextmenu', this.onContainerContextMenu, this);
        grid.un('headercontextmenu', this.onHeaderContextMenu, this);
        grid.un('itemcontextmenu', this.onItemContextMenu, this);

        Ext.destroyMembers(this, 'menu');
    },

    /**
     * Show the context menu when right clicking in an empty grid.
     *
     * @param {App.grid.Panel} grid
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onContainerContextMenu: function (grid, event, eOpts) {

        "use strict";

        event.stopEvent();
        this.protectBufferedStore();
        this.menu.showAt(event.getXY());
    },

    /**
     * Inhibit the context menu when right clicking in the grid header.
     *
     * @param {Ext.grid.header.Container} gridheader
     * @param {Ext.grid.column.Column} column
     * @param {Ext.EventObject} event
     * @param {HTMLElement} html
     * @param {Object} eOpts
     *
     */
    onHeaderContextMenu: function (gridheader, colum, event, html, eOpts) {

        "use strict";
        event.stopEvent();
    },

    /**
     * Show the context menu when right clicking in the non-empty grid.
     *
     * @param {Ext.view.View} view
     * @param {Ext.data.Model} record
     * @param {HTMLElement} item
     * @param {Ext.EventObject} event
     * @param {Object} eOpts
     *
     */
    onItemContextMenu: function (view, record, item, index, event, eOpts) {

        "use strict";

        event.stopEvent();
        this.protectBufferedStore();
        this.menu.showAt(event.getXY());
    },

    // private method to protected buffered store.
    // NOTE: in ExtJS 4.2.1 the editor operations crashed with a buffered store
    protectBufferedStore: function () {

        "use strict";
        var i,
            items,
            store = this.getCmp().getStore();

        if (store.buffered && Ext.getVersion('extjs').version === '4.2.1.883') {

            items = this.menu.query('menuitem');

            for (i = 0; i < items.length; i += 1)  {
                // separate menuitem from menuseparator
                if (items[i].getXType() === 'menuitem') {
                    items[i].setDisabled(true);
                }
            }
        }
    }
});