/**
 * A {alink Ext.panel.Panel panel} configure with the
 * {@link Ext.layout.container.Border border} layout splitting the space in
 * two parts. The first one contains the central panel and the second one
 * a selector.
 *
 * The selector is a collapsible panel appearing on the
 * left/right/top/bottom side of the central panel.
 * The selector contains a set of {@link Ext.form.Field fields} usually
 * organized in {@link Ext.form.FieldSet field sets}
 * and two {@link Ext.button.Button buttons} Go and Reset.
 *
 * The configuration parameter #selectorRegion defined the position
 * of the selector with respect to the central panel. Value are those
 * of the border layout: *east*, *west*, *south* and *north*.
 * By default the selector is on the east side (right).
 *
 * The size of the selector is determined by #selectorHeight and #selectorWidth.
 * The former works when the #selectorRegion is *north* and *south* while
 * the latter is for region *west* and *east*.
 *
 * The logic between the panel, the selector and the buttons
 * is defined in inherited class.
 *
 */
Ext.define('App.panel.BaseWithSelector', {

    extend: 'Ext.panel.Panel',
    alias: 'widget.xpanelwithselector',

    /**
     * @cfg {Object}
     * The main panel items.
     */
    panelItems: null,

    /**
     * @cfg {Boolean}
     */
    panelBorder: false,

    /**
     * @cfg {Object}
     */
    panelDefaults: null,

    /**
     * @cfg {Object}
     * The selector items.
     */
    selectorItems: null,

    /**
     * @cfg {Boolean}
     */
    selectorCollapsible: true,

    /**
     * @cfg {Object}
     */
    selectorDefaults: {layout: 'form', padding: '0 10 10 10'},

    /**
     * @cfg {Boolean}
     */
    selectorFrame: true,

    /**
     * @cfg {Number}
     */
    selectorHeight: 300,

    /**
     * @cfg {String}
     * Authorized values are east, west, south and north.
     */
    selectorRegion: 'east',

    /**
     * @cfg {Boolean}
     */
    selectorSplit: true,

    /**
     * @cfg {String}
     */
    selectorTitle: "Selector",

    /**
     * @cfg {Number}
     */
    selectorWidth: 300,

    /**
     * @property {Ext.Button}
     * The reference to the *go* button of the selector
     */
    goButton: null,

    /**
     * @property {Ext.Button}
     * The reference to the *reset* button of the selector
     */
    resetButton: null,

    /**
     * @property {Ext.panel.Panel}
     * reference to the mainPanel
     */
    mainPanel: null,

    /**
     * @property {Ext.panel.Panel}
     * reference to the selectorPanel
     */
    selectorPanel: null,

    // Private attribute for internationalization
    textGo: 'Go',
    textReset: 'Reset',

    // Private method requires by the ExtJS component model
    initComponent: function (config) {

        var cfg,
            toolbar;

        // apply the user configuration if any
        Ext.apply(this, config);

        // predefined configuration
        cfg = {
            layout: 'border',
            items: [{
                border: this.panelBorder,
                defaults: this.panelDefaults,
                itemId: 'mainPanel',
                items: this.panelItems,
                layout: 'fit',
                loader: {},
                plugins: ['pPanelLoaderException'],
                region: 'center'
            }, {
                bodyPadding: '0 5 0 0',
                buttons: [{
                    itemId: 'goButton',
                    text: this.textGo
                }, {
                    itemId: 'resetButton',
                    text: this.textReset
                }],
                collapsible: this.selectorCollapsible,
                defaults: this.selectorDefaults,
                frame: this.selectorFrame,
                height: this.selectorHeight,
                layout: 'form',
                itemId: 'selectorPanel',
                items: this.selectorItems,
                region: this.selectorRegion,
                split: this.selectorSplit,
                title: this.selectorTitle,
                width: this.selectorWidth
            }]
        };

        // apply the predefined configuration
        Ext.apply(this, cfg);

        // initialize the base class
        this.callParent(arguments);

        // short cuts
        this.mainPanel = this.getComponent('mainPanel');
        this.selectorPanel = this.getComponent('selectorPanel');

        // selector buttons
        toolbar = this.selectorPanel.getDockedItems()[0];
        this.goButton = toolbar.getComponent('goButton');
        this.resetButton = toolbar.getComponent('resetButton');
    }
});