/**
 * The Panel with URL selector is split in two parts the central panel
 * and the selector.
 * The selector display a list of {@link Ext.form.Field fields} organized in
 * {@link Ext.form.FieldSet field set}.
 * The URL content is displayed in the main panel.
 *
 * The URL display in the main panel is defined by the #baseUrl option.
 * It can be modified dynamically by extracting the name of the
 * *controller*, *function* and/or *extension* from the selector fields.
 * The options #ctrlField, #extField and #funcField are used to determine
 * the associated fields.
 *
 * Mathematical symbols will be interpreted when the javascript library
 * [MathJax](http://www.mathjax.org/) is load and when the #isMathJax
 * property is true.
 *
 */
Ext.define('App.panel.WithUrlSelector', {

    extend: 'App.panel.BaseWithSelector',
    alias: 'widget.xpanelwithurlselector',

    /**
     * @cfg {String}
     * The well-formed URL, e.g http://blabla.
     */
    baseUrl: undefined,

    /**
     * @cfg {String}
     * The name of the field defining the controller.
     * The URL becomes **baseUrl/ctrlFieldValue**.
     */
    ctrlField: undefined,

    /**
     * @cfg {String}
     * The name of the field defining the extension.
     * The URL becomes **baseUrl.extFieldValue**.
     */
    extField: undefined,

    /**
     * @cfg {String}
     * The name of the field defining the function.
     * The URL becomes **baseUrl/funcFieldValue**.
     */
    funcField: undefined,

    /**
     * @cfg {Boolean}
     * Set the value to true when the mathjax library is installed.
     */
    isMathJax: false,

    /**
     * @cfg {Number}
     * The time out for the {@link Ext.Ajax Ajax request} in millisecond.
     */
    timeout: 30000,

    // Private properties for internationalization
    textLoad: 'Loading...',
    textProgressBar: 'Preparing your file, please wait...',

    // Private method requires by the ExtJS model
    initComponent: function () {

        this.callParent(arguments);

        this.goButton.on('click', this.onGo, this);
        this.resetButton.on('click', this.onReset, this);
        this.mainPanel.getLoader().on('load', this.onLoad, this);
    },

    // Private method requires by the Ext JS component model
    beforeDestroy: function () {

        this.goButton.un('click', this.onGo, this);
        this.resetButton.un('click', this.onReset, this);
        this.mainPanel.getLoader().un('load', this.onLoad, this);

        this.callParent(arguments);
    },

    /**
     * handler to build the URL and to load it in the panel
     */
    onGo: function () {

        "use strict";

        var fieldName,
            fields,
            fName = "",
            i,
            iframe,
            params = {},
            tName = "",
            url = this.baseUrl,
            urlCtrl,
            urlFunc,
            urlExt;

        function extract(key, subfield) {
            params[subfield.getName()] = subfield.getValue();
        }

        // destroy embedded IFRAME in the mainPanel if any
        iframe = this.mainPanel.getEl().getById('mainPanelIframe');
        if (iframe) {
            Ext.destroy(iframe);
        }

        // activate the auto scrolling
        this.mainPanel.setAutoScroll(true);

        // extract basic parameters
        fields = this.selectorPanel.query('field');
        for (i = 0; i < fields.length; i += 1) {

            fieldName = fields[i].getName();

            // the field name is encode TableField
            [tName, fName] = App.decodeCapitalizeWords(fieldName);

            if (fName === this.ctrlField) {
                urlCtrl =  fields[i].getValue();

            } else if (fName === this.extField) {
                urlExt =  fields[i].getValue();

            } else if (fName === this.funcField) {
                urlFunc =  fields[i].getValue();

            } else {
                params[fieldName] = fields[i].getValue();
            }
        }

        // extract parameters embedded in composite fields
        fields = this.selectorPanel.query('fieldcontainer');
        for (i = 0; i < fields.length; i += 1) {
            fields[i].items.eachKey(extract);
        }

        // build the URL
        if (urlCtrl !== undefined) {
            url = url + '/' + urlCtrl;
        }

        if (urlFunc !== undefined) {
            url = url + '/' + urlFunc;
        }

        if (urlExt !== undefined) {
            url = url + '.' + urlExt;
        }

        // load the HTML content in the main panel
        if (!urlExt || urlExt === 'html' || urlExt === 'htm') {

            // inhibit the selector during loading
            this.selectorPanel.disable();

            this.mainPanel.getLoader().load({
                loadMask: true,
                params: params,
                text: this.textLoad,
                url: url,
                timeout: this.timeout
            });

        // Download the CSV, PDF, ... file using IFRAME
        } else {

            // launch a progress bar
            Ext.MessageBox.show({
                msg: this.textProgressBar,
                progressText: 'Saving...',
                width: 300,
                wait: true
            });

            // build the URL using the GET method
            url = url + '?' + Ext.Object.toQueryString(params);

            // append the IFRAME to the main panel therefore
            // the browser will launch the downloading.
            //
            // NOTE: the IFRAME cover almost the full space.
            // this is useful when the PDF is embedded in the panel.
            //
            // NOTE: keep the background colour to white
            //
            iframe = Ext.DomHelper.insertFirst(this.mainPanel.getEl(), {
                tag: 'iframe',
                id: 'mainPanelIframe',
                frameborder: 0,
                width: '100%',
                height: '99%',
                style: 'background: #FFFFFF;',
                src: url
            });

            // when the download is finish hide message box
            iframe = this.mainPanel.getEl().getById('mainPanelIframe');
            iframe.on('load', function () {
                Ext.MessageBox.hide();
            }, this, {single: true});
        }
    },

    /**
     * Handler call when the main panel is loaded.
     * Process symbols and enable the selector.
     *
     */

    onLoad: function () {

        "use strict";
        var domEl,
            loadMask;

        // process symbols
        if (MathJax && this.isMathJax) {

            domEl = Ext.getDom(this.mainPanel.getEl());

            loadMask = Ext.create('Ext.LoadMask', {
                target: this.mainPanel,
                msg: 'Processing symbols'
            });

            loadMask.show();

            MathJax.Hub.Queue(["Typeset", MathJax.Hub, domEl]);
            MathJax.Hub.Queue([function (mask) {mask.hide(); }, loadMask]);
        }

        // enable the selector
        this.selectorPanel.enable();
    },

    /**
     * Handler to reset the selector
     */
    onReset: function () {

        "use strict";

        var fields,
            i;

        fields = this.selectorPanel.query('field');
        for (i = 0; i < fields.length; i += 1) {
            fields[i].reset();
        }
    }
});